if not SimpleSelfRebuff then return end
local SimpleSelfRebuff = SimpleSelfRebuff
SimpleSelfRebuff:_rev("$Revision: 71645 $")

-------------------------------------------------------------------------------
-- Externals
-------------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("SimpleSelfRebuff")
local Tablet

local colorize = SimpleSelfRebuff.colorize
local formatDuration = SimpleSelfRebuff.formatDuration

local STATE_SET      = SimpleSelfRebuff.STATE_SET
local STATE_FOUND    = SimpleSelfRebuff.STATE_FOUND
local STATE_EXPIRING = SimpleSelfRebuff.STATE_EXPIRING
local STATE_QUIET    = SimpleSelfRebuff.STATE_QUIET
local STATE_DONTCAST = SimpleSelfRebuff.STATE_DONTCAST

local bit_band = bit.band
local bit_bor = bit.bor

-------------------------------------------------------------------------------
-- Helpers
-------------------------------------------------------------------------------

local function GetBuffInfo(category, state, expected, actual, timeLeft)
	if bit_band(state or 0, STATE_SET) == 0 then
		return nil
	elseif bit_band(state, STATE_DONTCAST) ~= 0 then
		return expected.name, L["Ignored"], 1, 0.5, 0
	elseif bit_band(state, STATE_FOUND) == 0 then
		return expected.name, L["Missing"], 1, 0, 0
	else
		timeLeft = formatDuration(timeLeft, "-")
		if expected == actual then
			return actual.name, timeLeft, 0, 1, 0
		else
			return actual.name, timeLeft, 1, 1, 0
		end
	end
end

-------------------------------------------------------------------------------
-- Module declaration
-------------------------------------------------------------------------------

local FuBarPlugin = SimpleSelfRebuff:NewModule("FuBarPlugin", "FuBarPlugin-2.0")

FuBarPlugin.title = SimpleSelfRebuff.name

FuBarPlugin.OnMenuRequest = SimpleSelfRebuff.options
FuBarPlugin.options = AceLibrary("FuBarPlugin-2.0"):GetAceOptionsDataTable(FuBarPlugin)
FuBarPlugin.optionName = L["FuBar settings"]
FuBarPlugin.optionOrder = -4

FuBarPlugin.hasIcon = 'Interface\\Icons\\Spell_Holy_LayOnHands'
 
function FuBarPlugin:OnModuleInitialize()
	self.version = SimpleSelfRebuff.version
	self.revision = SimpleSelfRebuff.revision		
end

function FuBarPlugin:OnModuleEnable()
	self:RegisterBucketEvent({'SimpleSelfRebuff_UpdateDisplay', 'SimpleSelfRebuff_StateChanged'}, 0.1, 'UpdateDisplay')
end

function FuBarPlugin:OnTextUpdate()
	local active = self.core:IsMonitoringActive()
	if active then
		self:SetText('SSR')
	else
		self:SetText('|cff777777SSR|r')
	end
	self:SetIcon(self.hasIcon)
	for name, module in self.core:IterateModulesWithMethod('OnTextUpdate') do
		if module ~= self then
			module:OnTextUpdate(self)
		end
	end	
end

function FuBarPlugin:OnTooltipUpdate()
	if not Tablet then
		Tablet = AceLibrary('Tablet-2.0')
	end
	
	local active = self.core:IsMonitoringActive()
	local cat
	
	Tablet:SetTitle(self.core.name)
	
	if not active then
		cat = Tablet:AddCategory()
		cat:AddLine('text', L["Currently inactive (resting/dead)."])
		
	else
		cat = Tablet:AddCategory("columns", 3)
		for category, state, expected, actual, timeLeft in self.core:IterateCategories(STATE_SET) do
			local name, timeLeft, r, g, b = GetBuffInfo(category, state, expected, actual, timeLeft)
			if self.core:IsDebugging() then
				timeLeft = ("state=%s, expected=%s, actual=%s, timeLeft=%s"):format(
					self.core:fmtState(state), 
					expected and expected.name or "<nil>", 
					actual and actual.name or "<nil>",
					timeLeft or "<nil>"
				)
				name = name or "-"
			end
			if name then
				cat:AddLine(
					'text',  category.name,
					'text2', name,     'text2R', r, 'text2G', g, 'text2B', b,
					'text3', timeLeft, 'text3R', r, 'text3G', g, 'text3B', b
				)
			end
		end
		self:ScheduleEvent('SimpleSelfRebuff_UpdateDisplay', 1)		
		Tablet:SetHint(L["Green means \"ok\", yellow means \"override wanted buff\", red means \"missing\"."])
	end
	
	for name, module in self.core:IterateModulesWithMethod('OnTooltipUpdate') do
		if module ~= self then
			module:OnTooltipUpdate(Tablet)
		end
	end
	
end
